==> GotoBLAS-1.15.tar.gz: GotoBLAS/lapack/lauum/lauum_U_single.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include "common.h"

static FLOAT dp1 =  1.;

#ifndef COMPLEX
#define TRMM_KERNEL	TRMM_KERNEL_RT
#define SYRK_KERNEL	SYRK_KERNEL_U
#else
#define TRMM_KERNEL	TRMM_KERNEL_RC
#ifdef XDOUBLE
#define SYRK_KERNEL	xherk_kernel_UN
#elif defined(DOUBLE)
#define SYRK_KERNEL	zherk_kernel_UN
#else
#define SYRK_KERNEL	cherk_kernel_UN
#endif
#endif

#define GEMM_PQ  MAX(GEMM_P, GEMM_Q)
#define REAL_GEMM_R (GEMM_R - GEMM_PQ)

blasint CNAME(long  n, FLOAT *a, long lda, FLOAT *sa, FLOAT *sb){

  long i, bk, blocking;
  long is, ls, ks;
  long min_i, min_l, min_k;

  FLOAT *sb2 = (FLOAT *)((((long)sb
		    + GEMM_PQ  * GEMM_Q * COMPSIZE * SIZE + GEMM_ALIGN) & ~GEMM_ALIGN)
		  + GEMM_BUFFER_B_OFFSET);

  FLOAT *sub = (FLOAT *)alloca(GEMM_UNROLL_M * GEMM_UNROLL_N * COMPSIZE * SIZE);

#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
  FLOAT *aa;
#endif

  if (n <= DTB_ENTRIES) {
    LAUU2_U(n, a, lda, sb);
    return 0;
  }

  blocking = GEMM_Q;
  if (n <= 4 * GEMM_Q) blocking = n / 4;

  for (i = 0; i < n; i += blocking) {

    bk = MIN(blocking, n - i);
    
    if (i > 0) {

      TRMM_OUTCOPY(bk, bk, a + (i + i * lda) * COMPSIZE, lda, 0, 0, sb);

      for (ls = 0; ls < i; ls += REAL_GEMM_R) {
	min_l = i - ls;
	if (min_l > REAL_GEMM_R) min_l = REAL_GEMM_R;

	GEMM_OTCOPY(bk,  min_l,  a + (ls  + i * lda) * COMPSIZE, lda, sb2);
	
	for(is = 0; is < ls + min_l ; is += GEMM_P){
	  min_i = ls + min_l - is;
	  if (min_i > GEMM_P) min_i = GEMM_P;
	  
#if  (GEMM_UNROLL_M == GEMM_UNROLL_N)
	  if (is < ls) {
	    GEMM_ITCOPY(bk, min_i, a + (is + i * lda) * COMPSIZE, lda, sa);
	    aa = sa;
	  } else {
	    aa = sb2 + bk * (is - ls) * COMPSIZE;
	  }
#else
	  GEMM_ITCOPY(bk, min_i, a + (is + i * lda) * COMPSIZE, lda, sa);
#endif
	  
	  SYRK_KERNEL(min_i, min_l, bk, dp1, 
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
		      aa, 
#else
		      sa, 
#endif
		      sb2, 
		      a + (is + ls * lda) * COMPSIZE, lda, is - ls, sub);
	  
	  if (ls + REAL_GEMM_R >= i ) {
	    for (ks = 0; ks < bk; ks += GEMM_P) {
	      min_k = bk - ks;
	      if (min_k > GEMM_P) min_k = GEMM_P;
	      
	      TRMM_KERNEL(min_i, min_k, bk, dp1,
#ifdef COMPLEX
			  ZERO,
#endif
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
			  aa, 
#else
			  sa,
#endif
			  sb + ks * bk * COMPSIZE,
			  a + (is + (ks + i) * lda) * COMPSIZE, lda, -ks); 
	    }
	  }
	} /* end of is */
      } /* end of ls */
    }

    CNAME(bk, a + (i + i * lda) * COMPSIZE, lda, sa, sb);

  }

  return 0;
}

==> GotoBLAS-1.19.tar.gz: GotoBLAS/lapack/lauum/lauum_U_single.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include "common.h"

static FLOAT dp1 =  1.;

#ifndef COMPLEX
#define TRMM_KERNEL	TRMM_KERNEL_RT
#define SYRK_KERNEL	SYRK_KERNEL_U
#else
#define TRMM_KERNEL	TRMM_KERNEL_RC
#ifdef XDOUBLE
#define SYRK_KERNEL	xherk_kernel_UN
#elif defined(DOUBLE)
#define SYRK_KERNEL	zherk_kernel_UN
#else
#define SYRK_KERNEL	cherk_kernel_UN
#endif
#endif

#define GEMM_PQ  MAX(GEMM_P, GEMM_Q)
#define REAL_GEMM_R (GEMM_R - GEMM_PQ)
#define MAX_MN	MAX(GEMM_UNROLL_M, GEMM_UNROLL_N)

blasint CNAME(long  n, FLOAT *a, long lda, FLOAT *sa, FLOAT *sb){

  long i, bk, blocking;
  long is, ls, ks;
  long min_i, min_l, min_k;

  FLOAT *sb2 = (FLOAT *)((((long)sb
		    + GEMM_PQ  * GEMM_Q * COMPSIZE * SIZE + GEMM_ALIGN) & ~GEMM_ALIGN)
		  + GEMM_BUFFER_B_OFFSET);

  FLOAT *sub = (FLOAT *)alloca(MAX_MN * MAX_MN * COMPSIZE * SIZE);

#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
  FLOAT *aa;
#endif

  if (n <= DTB_ENTRIES) {
    LAUU2_U(n, a, lda, sb);
    return 0;
  }

  blocking = GEMM_Q;
  if (n <= 4 * GEMM_Q) blocking = n / 4;

  for (i = 0; i < n; i += blocking) {

    bk = MIN(blocking, n - i);
    
    if (i > 0) {

      TRMM_OUTCOPY(bk, bk, a + (i + i * lda) * COMPSIZE, lda, 0, 0, sb);

      for (ls = 0; ls < i; ls += REAL_GEMM_R) {
	min_l = i - ls;
	if (min_l > REAL_GEMM_R) min_l = REAL_GEMM_R;

	GEMM_OTCOPY(bk,  min_l,  a + (ls  + i * lda) * COMPSIZE, lda, sb2);
	
	for(is = 0; is < ls + min_l ; is += GEMM_P){
	  min_i = ls + min_l - is;
	  if (min_i > GEMM_P) min_i = GEMM_P;
	  
#if  (GEMM_UNROLL_M == GEMM_UNROLL_N)
	  if (is < ls) {
	    GEMM_ITCOPY(bk, min_i, a + (is + i * lda) * COMPSIZE, lda, sa);
	    aa = sa;
	  } else {
	    aa = sb2 + bk * (is - ls) * COMPSIZE;
	  }
#else
	  GEMM_ITCOPY(bk, min_i, a + (is + i * lda) * COMPSIZE, lda, sa);
#endif
	  
	  SYRK_KERNEL(min_i, min_l, bk, dp1, 
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
		      aa, 
#else
		      sa, 
#endif
		      sb2, 
		      a + (is + ls * lda) * COMPSIZE, lda, is - ls, sub);
	  
	  if (ls + REAL_GEMM_R >= i ) {
	    for (ks = 0; ks < bk; ks += GEMM_P) {
	      min_k = bk - ks;
	      if (min_k > GEMM_P) min_k = GEMM_P;
	      
	      TRMM_KERNEL(min_i, min_k, bk, dp1,
#ifdef COMPLEX
			  ZERO,
#endif
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
			  aa, 
#else
			  sa,
#endif
			  sb + ks * bk * COMPSIZE,
			  a + (is + (ks + i) * lda) * COMPSIZE, lda, -ks); 
	    }
	  }
	} /* end of is */
      } /* end of ls */
    }

    CNAME(bk, a + (i + i * lda) * COMPSIZE, lda, sa, sb);

  }

  return 0;
}
